/*
 * System Interface Library for games
 * Copyright (c) 2007-2014 Andrew Church <achurch@achurch.org>
 * Released under the GNU GPL version 3 or later; NO WARRANTY is provided.
 * See the file COPYING.txt for details.
 *
 * src/sysdep/test/sound.c: Testing implementation of the system-level
 * audio output interface.
 */

#ifdef INCLUDE_TESTS  // To the end of the file.

#define IN_SYSDEP_TEST

#include "src/base.h"
#include "src/math.h"
#include "src/memory.h"
#include "src/sound.h"
#include "src/sound/mixer.h"
#include "src/sysdep.h"
#include "src/sysdep/test.h"

/*************************************************************************/
/****************************** Local data *******************************/
/*************************************************************************/

/* Sampling rate to use for mixing. */
static int output_rate = 48000;

/* Maximum number of channels to allow in sys_sound_init() (0 = unlimited). */
static int max_channels = 0;

/* Have we been initialized? */
static uint8_t sound_initted = 0;

/* Should sys_sound_setdata() always fail? */
static uint8_t fail_setdata = 0;

/* Should we report a headphone disconnect? */
static uint8_t headphone_disconnect_flag;

/*************************************************************************/
/************************** Interface routines ***************************/
/*************************************************************************/

int sys_sound_init(const char *device_name, int num_channels,
                   int *channels_ret)
{
    PRECOND(device_name != NULL, return 0);
    PRECOND(num_channels > 0, return 0);
    PRECOND(channels_ret != NULL, return 0);
    PRECOND(!sound_initted, return 0);

    /* Magic value used in sound core tests. */
    if (strcmp(device_name, "FAIL") == 0) {
        DLOG("Failing as requested");
        return 0;
    }

    if (max_channels > 0 && num_channels > max_channels) {
        DLOG("Limiting to %d channels as requested", max_channels);
        num_channels = max_channels;
    }

    if (!sound_mixer_init(num_channels, output_rate)) {
        DLOG("Failed to initialize software mixer!");
        return 0;
    }
    for (int i = 0; i < num_channels; i++) {
        channels_ret[i] = i+1;
    }

    /* Magic values used in sound core tests. */
    if (strcmp(device_name, "NEGA") == 0) {
        DLOG("Setting output_rate to -1 as requested");
        output_rate = -1;  // Will cause sound_init() to fail.
    } else if (strcmp(device_name, "ZERO") == 0) {
        DLOG("Setting output_rate to 0 as requested");
        output_rate = 0;  // Will disable decoder resampling.
    }

    sound_initted = 1;
    return num_channels;
}

/*-----------------------------------------------------------------------*/

int sys_sound_playback_rate(void)
{
    return output_rate;
}

/*-----------------------------------------------------------------------*/

int sys_sound_check_headphone_disconnect(void)
{
    return headphone_disconnect_flag;
}

/*-----------------------------------------------------------------------*/

void sys_sound_acknowledge_headphone_disconnect(void)
{
    headphone_disconnect_flag = 0;
}

/*-----------------------------------------------------------------------*/

void sys_sound_cleanup(void)
{
    PRECOND(sound_initted, return);

    sound_mixer_cleanup();
    sound_initted = 0;
}

/*-----------------------------------------------------------------------*/

/*
 * The remaining sys_sound_*() functions are all redirected to the software
 * mixer functions, which implement the actual channel management.
 */

int sys_sound_setdata(int channel, SoundDecodeCallback decode_func,
                      void *handle, int is_stereo, UNUSED int sample_rate)
{
    if (fail_setdata) {
        DLOG("Failing as requested");
        return 0;
    }
    return sound_mixer_setdata(channel, decode_func, handle, is_stereo);
}

void sys_sound_setvol(int channel, float volume)
{
    return sound_mixer_setvol(channel, volume);
}

void sys_sound_setpan(int channel, float pan)
{
    return sound_mixer_setpan(channel, pan);
}

void sys_sound_setfade(int channel, float target, float length, int cut)
{
    return sound_mixer_setfade(channel, target, length, cut);
}

void sys_sound_start(int channel)
{
    return sound_mixer_start(channel);
}

void sys_sound_stop(int channel)
{
    return sound_mixer_stop(channel);
}

void sys_sound_reset(int channel)
{
    return sound_mixer_reset(channel);
}

int sys_sound_status(int channel)
{
    return sound_mixer_status(channel);
}

/*************************************************************************/
/************************* Test control routines *************************/
/*************************************************************************/

void sys_test_sound_set_output_rate(int rate)
{
    PRECOND(!sound_initted, return);
    output_rate = rate;
}

/*-----------------------------------------------------------------------*/

void sys_test_sound_set_max_channels(int channels)
{
    PRECOND(!sound_initted, return);
    max_channels = channels;
}

/*-----------------------------------------------------------------------*/

void sys_test_sound_set_fail_setdata(int fail)
{
    fail_setdata = fail;
}

/*-----------------------------------------------------------------------*/

#ifdef SIL_SOUND_HEADPHONE_CHECK
void sys_test_sound_set_headphone_disconnect(void)
{
    headphone_disconnect_flag = 1;
}
#endif

/*-----------------------------------------------------------------------*/

void sys_test_sound_get_pcm(int16_t *buffer, int samples)
{
    sound_mixer_get_pcm(buffer, samples);
}

/*************************************************************************/
/*************************************************************************/

#endif  // INCLUDE_TESTS

/*
 * Local variables:
 *   c-file-style: "stroustrup"
 *   c-file-offsets: ((case-label . *) (statement-case-intro . *))
 *   indent-tabs-mode: nil
 * End:
 *
 * vim: expandtab shiftwidth=4:
 */
