/*
 * System Interface Library for games
 * Copyright (c) 2007-2014 Andrew Church <achurch@achurch.org>
 * Released under the GNU GPL version 3 or later; NO WARRANTY is provided.
 * See the file COPYING.txt for details.
 *
 * src/test/movie/webm.c: Movie functionality tests specific to WebM
 * software decoding.
 */

#ifdef INCLUDE_TESTS  // To the end of the file.

#include "src/base.h"
#include "src/graphics.h"
#include "src/movie.h"
#include "src/movie/webm.h"
#include "src/sound.h"
#include "src/sysdep.h"
#include "src/sysdep/test.h"
#include "src/test/base.h"
#include "src/test/graphics/internal.h"
#include "src/texture.h"

#ifndef SIL_MOVIE_INCLUDE_WEBM
int test_movie_webm(void) {
    DLOG("WebM support disabled, nothing to test.");
    return 1;
}
#else  // To the end of the file.

/*************************************************************************/
/***************************** Test routines *****************************/
/*************************************************************************/

static int do_test_movie_webm(void);
int test_movie_webm(void)
{
    return run_tests_in_window(do_test_movie_webm);
}

DEFINE_GENERIC_TEST_RUNNER(do_test_movie_webm)

TEST_INIT(init)
{
    sys_test_sound_set_output_rate(44100);
    sys_test_sound_set_max_channels(0);
    sound_init();
    CHECK_INTEQUAL(sound_open_device("", 2), 2);
    return 1;
}

TEST_CLEANUP(cleanup)
{
    graphics_flush_resources();
    sound_cleanup();
    return 1;
}

/*-----------------------------------------------------------------------*/

TEST(test_play_sound_decoder_failure)
{
    int movie;
    CHECK_TRUE(movie = movie_open("testdata/movie/test.webm", 0, 0));

    /* The movie will have reserved sound channel 1 for playback.  We free
     * it here so the sound_play_decoder() call in core.c/movie_play()
     * will fail. */
    sound_free_channel(1);
    CHECK_FALSE(movie_play(movie));

    movie_close(movie);
    return 1;
}

/*-----------------------------------------------------------------------*/

TEST(test_draw_texture_lock_failure)
{
    int movie;
    CHECK_TRUE(movie = movie_open("testdata/movie/test.webm", 0, 0));
    CHECK_TRUE(movie_play(movie));

    /* The movie will have created a texture with ID 1 for the video image.
     * We lock it here so the lock call in core.c/movie_draw() will
     * fail. */
    ASSERT(texture_lock(1));
    CHECK_FALSE(movie_draw(movie));

    movie_close(movie);
    return 1;
}

/*-----------------------------------------------------------------------*/

TEST(test_play_audio_end_of_stream)
{
    int movie;
    CHECK_TRUE(movie = movie_open("testdata/movie/test.webm", 0, 0));

    CHECK_TRUE(movie_play(movie));
    const int audio_length = (44100/30) * 20
        + 1000;  // There will be a few extra samples at the end of the stream.
    int16_t pcm[1024*2];
    for (int samples = 0; samples < audio_length; samples += lenof(pcm)/2) {
        sys_test_sound_get_pcm(pcm, lenof(pcm)/2);
    }
    /* We can't directly check whether audio playback has terminated, so
     * here we just check that the audio output is silent, and we rely on
     * branch coverage data to determine that the end-of-stream branch
     * has actually been taken. */
    sys_test_sound_get_pcm(pcm, 1);
    CHECK_INTEQUAL(pcm[0], 0);
    CHECK_INTEQUAL(pcm[1], 0);

    /* Video playback should keep going even after the audio is done. */
    CHECK_TRUE(movie_is_playing(movie));

    movie_close(movie);
    return 1;
}

/*-----------------------------------------------------------------------*/

TEST(test_play_broken_video)
{
    int movie;
    CHECK_TRUE(movie = movie_open("testdata/movie/broken-video.webm", 0, 0));

    CHECK_TRUE(movie_play(movie));
    CHECK_FALSE(movie_next_frame(movie));  // Frame data is corrupt.
    CHECK_FALSE(movie_is_playing(movie));  // Error should terminate playback.

    movie_close(movie);
    return 1;
}

/*-----------------------------------------------------------------------*/

TEST(test_play_broken_audio)
{
    int movie;
    CHECK_TRUE(movie = movie_open("testdata/movie/broken-audio.webm", 0, 0));

    CHECK_TRUE(movie_play(movie));
    int16_t pcm[20];
    sys_test_sound_get_pcm(pcm, lenof(pcm)/2);
    for (int i = 0; i < lenof(pcm); i++) {
        CHECK_INTEQUAL(pcm[i], 0);
    }
    CHECK_TRUE(movie_is_playing(movie));  // Video playback should keep going.

    movie_close(movie);
    return 1;
}

/*-----------------------------------------------------------------------*/

TEST(test_play_audio_memory_failures)
{
    WebMDecodeHandle *handle;
    int16_t pcm[2];

    /* We don't use CHECK_MEMORY_FAILURES here for the same reason we don't
     * use it in the core memory_failures test: the Vorbis decoder makes
     * numerous allocations at initialization time, and it would take a
     * long time to iterate through them all.  Instead, we loop on just the
     * get_audio call until success and rely on the leak check for the test
     * itself to catch memory leaks. */
    for (int try = 0; ; try++) {
        if (try >= 100) {
            FAIL("movie_webm_get_audio(handle, pcm, lenof(pcm)) did not"
                 " succeed after %d iterations", try);
        }
        SysFile *fh;
        ASSERT(fh = sys_file_open("testdata/movie/test-mono.webm"));
        CHECK_TRUE(handle = movie_webm_open(fh, 0, sys_file_size(fh)));
        CHECK_INTEQUAL(movie_webm_audio_channels(handle), 1);

        TEST_mem_fail_after(try, 1);
        const int result = movie_webm_get_audio(handle, pcm, lenof(pcm));
        TEST_mem_fail_after(-1, 0);
        if (result) {
            break;
        }
        movie_webm_close(handle);
    }

    /* We seem to get slightly different output here, perhaps because the
     * memory failure pushes libvorbis into a different mode. */
    CHECK_INTRANGE(pcm[0], 0x40, 0x50);
    CHECK_INTRANGE(pcm[1], 0x2AC, 0x2BC);

    /* A subsequent get_audio call should still succeed, though it will
     * have skipped data. */
    CHECK_INTEQUAL(movie_webm_get_audio(handle, pcm, lenof(pcm)), lenof(pcm));

    movie_webm_close(handle);
    return 1;
}

/*************************************************************************/
/*************************************************************************/

#endif  // SIL_MOVIE_INCLUDE_WEBM
#endif  // INCLUDE_TESTS

/*
 * Local variables:
 *   c-file-style: "stroustrup"
 *   c-file-offsets: ((case-label . *) (statement-case-intro . *))
 *   indent-tabs-mode: nil
 * End:
 *
 * vim: expandtab shiftwidth=4:
 */
